/*******************************************************************************
********************************************************************************

   COPYRIGHT  2004, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   ap_headervalid.c	   Application Header Validation Function C Source File

   About:         This module contains the application header validation functions
                  for the AICONS01 project.

   Target CPU:    H8S/2215

   Written for:   Advanced Illumination, Inc.
                  24 Peavine Drive
                  Rochester, VT 05767
                  http://advancedillumination.com

   Contacts:      John Thrailkill
                  Phone:  (802) 767-3830
                  FAX:    (802) 767-3831
                  jthrailkill@advill.com

   Written by:    Rob Macklin
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 1 $

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/Pulsar320/tar_src/H8App/ap_headervalid.c $
 * 
 * 1     8/15/08 8:45a Jpiet
 * Initial check-in
 * 
 * 3     1/25/06 9:01a Rmack
 * Changed define used to indicate PC build.
 * 
 * 2     2/09/05 2:39p Rmack
 * Added AI storage data structure functionality.
 * 
 * 1     9/23/04 10:50a Rmack
 * New module to handle all validation functions.
 *
********************************************************************************
*******************************************************************************/
#include "compiler.h"
#include "tk_crc.h"

#ifndef _MSC_VER
#include "ap_header.h"
#include "tk_flash.h"

APHEADER_APP_STRUCT_T * const apheader_header_search_addresses[] = 
{
   (APHEADER_APP_STRUCT_T *) TKFLASH_64K_BLOCK0_ADDRESS,
   (APHEADER_APP_STRUCT_T *) TKFLASH_64K_BLOCK1_ADDRESS,
   (APHEADER_APP_STRUCT_T *) TKFLASH_64K_BLOCK2_ADDRESS
};


const UINT8_T apheader_num_search_addresses = LENGTH(apheader_header_search_addresses);


#else
#pragma pack(2)
#include "ap_header.h"
#pragma pack()

#endif /* WIN32 */

/*******************************************************************************
   apheader_app_info_valid

   About:      

   Inputs:     
   
   Outputs:

   Returns:    

********************************************************************************/
BOOLEAN_T apheader_app_info_valid(APHEADER_APP_STRUCT_T * hdr)
{
   BOOLEAN_T valid = FALSE;
   UINT32_T crc_len;
   UINT16_T crc;
   
   if (SWAP_UINT16_M(hdr->struct_type) == APHEADER_APP_TYPE)
   {
      if (strcmp(APHEADER_ID_STRING,(INT8_T *) hdr->fw_id) == 0)
      {
         /* Determine the length of bytes for the structure CRC value */
         /* Include all bytes in the structure except the CRC itself  */
         crc_len = sizeof(APHEADER_APP_STRUCT_T) - sizeof(UINT16_T);

         /* Calculate the CRC-16 value for the structure */
         TKCRC_CALC16_M((UINT8_T *) hdr + sizeof(UINT16_T) ,crc_len,&crc);
      
         if (crc == SWAP_UINT16_M(hdr->struct_crc))
         {
            valid = TRUE;
         }
      }
   }

   return valid;
}

/*******************************************************************************
   apheader_manufacture_info_valid

   About:      

   Inputs:     
   
   Outputs:

   Returns:    

********************************************************************************/
BOOLEAN_T apheader_manufacture_info_valid(APHEADER_MANUFACTURE_STRUCT_T *hdr)
{
   BOOLEAN_T valid = FALSE;
   UINT32_T crc_len;
   UINT16_T crc;
   
   if (SWAP_UINT16_M(hdr->struct_type) == APHEADER_MANUFACTURE_TYPE)
   {
      /* Determine the length of bytes for the structure CRC value */
      /* Include all bytes in the structure except the CRC itself  */
      crc_len = sizeof(APHEADER_MANUFACTURE_STRUCT_T) - sizeof(UINT16_T);
//      crc_len = (UINT16_T) ((UINT32_T) hdr->notes - (UINT32_T) hdr->serial_num);

      /* Calculate the CRC-16 value for the structure */
      TKCRC_CALC16_M((UINT8_T *) hdr + sizeof(UINT16_T) ,crc_len,&crc);

      /* If computed CRC agrees with the structure CRC */
      if (crc == SWAP_UINT16_M(hdr->struct_crc))
      {
         valid = TRUE;
      }
   }

   return valid;

}

/*******************************************************************************
   apheader_cal_info_valid

   About:      

   Inputs:     
   
   Outputs:

   Returns:    

********************************************************************************/
BOOLEAN_T apheader_cal_info_valid(APHEADER_CAL_STRUCT_T *hdr)
{
   BOOLEAN_T valid = FALSE;
   UINT32_T crc_len;
   UINT16_T crc;
   
   if (SWAP_UINT16_M(hdr->struct_type) == APHEADER_CAL_TYPE)
   {
      /* Determine the length of bytes for the structure CRC value */
      /* Include all bytes in the structure except the CRC itself  */
      crc_len = sizeof(APHEADER_CAL_STRUCT_T) - sizeof(UINT16_T);

      /* Calculate the CRC-16 value for the structure */
      TKCRC_CALC16_M((UINT8_T *) hdr + sizeof(UINT16_T) ,crc_len,&crc);

      /* If computed CRC agrees with the structure CRC */
      if (crc == SWAP_UINT16_M(hdr->struct_crc))
      {
         valid = TRUE;
      }
   }

   return valid;

}

/*******************************************************************************
   apheader_ai_storage_info_valid

   About:      

   Inputs:     
   
   Outputs:

   Returns:    

********************************************************************************/
BOOLEAN_T apheader_ai_storage_info_valid(APHEADER_AI_STORAGE_STRUCT_T * hdr)
{
   BOOLEAN_T valid = FALSE;
   UINT32_T crc_len;
   UINT16_T crc;
   
   if (SWAP_UINT16_M(hdr->struct_type) == APHEADER_AI_STORAGE_TYPE)
   {
      /* Determine the length of bytes for the structure CRC value */
      /* Include all bytes in the structure except the CRC itself  */
      crc_len = sizeof(APHEADER_AI_STORAGE_STRUCT_T) - sizeof(UINT16_T);

      /* Calculate the CRC-16 value for the structure */
      TKCRC_CALC16_M((UINT8_T *) hdr + sizeof(UINT16_T) ,crc_len,&crc);

      /* If computed CRC agrees with the structure CRC */
      if (crc == SWAP_UINT16_M(hdr->struct_crc))
      {
         valid = TRUE;
      }
   }

   return valid;

}


/*******************************************************************************
   apheader_config_info_valid

   About:      

   Inputs:     
   
   Outputs:

   Returns:    

********************************************************************************/
BOOLEAN_T apheader_config_info_valid(APHEADER_CONFIG_STRUCT_T *hdr)
{
   BOOLEAN_T valid = FALSE;
   UINT32_T crc_len;
   UINT16_T crc;
   
   if (SWAP_UINT16_M(hdr->struct_type) == APHEADER_CONFIG_TYPE)
   {
      /* Determine the length of bytes for the structure CRC value */
      /* Include all bytes in the structure except the CRC itself  */
      crc_len = sizeof(APHEADER_CONFIG_STRUCT_T) - sizeof(UINT16_T);

      /* Calculate the CRC-16 value for the structure */
      TKCRC_CALC16_M((UINT8_T *) hdr + sizeof(UINT16_T) ,crc_len,&crc);

      /* If computed CRC agrees with the structure CRC */
      if (crc == SWAP_UINT16_M(hdr->struct_crc))
      {
         valid = TRUE;
      }
   }

   return valid;

}



/*******************************************************************************
   apheader_lighthead_info_valid

   About:      

   Inputs:     
   
   Outputs:

   Returns:    

********************************************************************************/
BOOLEAN_T apheader_lighthead_info_valid(APHEADER_LIGHTHEAD_STRUCT_T *hdr)
{
   BOOLEAN_T valid = FALSE;
   UINT32_T crc_len;
   UINT16_T crc;
   
   if (SWAP_UINT16_M(hdr->struct_type) == APHEADER_LIGHTHEAD_TYPE)
   {
      /* Determine the length of bytes for the structure CRC value */
      /* Include all bytes in the structure except the CRC itself  */
      crc_len = sizeof(APHEADER_LIGHTHEAD_STRUCT_T) - sizeof(UINT16_T);

      /* Calculate the CRC-16 value for the structure */
      TKCRC_CALC16_M((UINT8_T *) hdr + sizeof(UINT16_T) ,crc_len,&crc);

      /* If computed CRC agrees with the structure CRC */
      if (crc == SWAP_UINT16_M(hdr->struct_crc))
      {
         valid = TRUE;
      }
   }

   return valid;

}



/*******************************************************************************
   apheader_app_code_valid

   About:      

   Inputs:     
   
   Outputs:

   Returns:    

********************************************************************************/
BOOLEAN_T apheader_app_code_valid(APHEADER_APP_STRUCT_T * hdr)
{
   BOOLEAN_T valid = FALSE;
   UINT16_T crc;
   
   if (SWAP_UINT16_M(hdr->struct_type) == APHEADER_APP_TYPE)
   {
      /* Calculate application CRC */
      TKCRC_CALC16_M((UINT8_T *) hdr->app_start,
                     (UINT32_T) hdr->app_end - (UINT32_T) hdr->app_start,
                     &crc);
      
      /* Check if secure code CRC matches header information */
      if (SWAP_UINT16_M(hdr->app_crc) == crc)
      {
         /* All code is valid */
         valid = TRUE;
      }
   }

   return valid;
}

