/*******************************************************************************
********************************************************************************

   COPYRIGHT  2004, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   tk_pulse.h     Pulse Toolkit C Header File

   About:         This module contains the Pulse Toolkit
                  routines for the AICONS01 project.

   Target CPU:    H8S/2215

   Written for:   Advanced Illumination, Inc.
                  24 Peavine Drive
                  Rochester, VT 05767
                  http://advancedillumination.com

   Contacts:      John Thrailkill
                  Phone:  (802) 767-3830
                  FAX:    (802) 767-3831
                  jthrailkill@advill.com

   Written by:    Rob Macklin
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 2 $

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/Pulsar320/pc_src/LEDTest/tk_pulse.h $
 * 
 * 2     8/15/08 1:19p Jpiet
 * Added TKPULSE_NUM_CHAN to WIN32 build.
 * 
 * 1     8/15/08 8:45a Jpiet
 * Initial check-in
 * 
 * 19    5/14/08 5:12p Rmack
 * Changes to support 32-bit width parameter.
 * 
 * 18    4/05/07 1:07p Rmack
 * Defines for max LV supply limit.
 * 
 * 17    12/30/05 3:41p Rmack
 * Changed default max_lv_volts.
 * 
 * 16    4/28/05 12:32p Rmack
 * Defined new gated continuous mode of operation.
 * 
 * 15    2/17/05 4:20p Rmack
 * New define for internal trigger frequency
 * 
 * 12    10/13/04 9:13a Rmack
 * New functions defined, updates to constants.
 * 
 * 11    9/21/04 11:58a Rmack
 * 
 * 10    9/13/04 10:21a Rmack
 * Added commands for direct control of pulse outputs.
 * 
 * 7     9/07/04 12:22p Rmack
 * Pulse control data structures in place.  Support for trigger mapping,
 * frequency thresholding, variable maximum frequency, trigger edge
 * selection.
 * 
 * 6     8/24/04 8:03a Rmack
 * DTC based triggering of all four channels with  <  2usec delay.  Basic
 * TK_PULSE command interface implemented.
 * 
 * 5     8/17/04 4:50p Rmack
 * Restructure of existing code, release for beta test site.  Still ISR
 * driven triggering.
 * 
 * 4     8/11/04 4:40p Jwayg
 * Final check-in under JWayg name, code currently setup to run LED tests.
 * 
 * 3     7/01/04 10:30a Jwayg
 * Last code release before transfer of source files to new programmer.
 * 
 * 2     6/11/04 10:58a Jwayg
 * 8200-000068-005 - AI Trade Show Build
 * 
 * 1     6/07/04 9:57a Jwayg
 * S1000 - -005 firmware level - pulses created programatically via
 * special commands
 * 
 * 2     6/01/04 5:05p Jwayg
 * S1000 - in process version with test pulses - being geared for Jon's
 * hardware testing - 8200-000068-003 level
 * 
 * 1     5/17/04 10:59a Jwayg
 * S1000 application specific files
 * 
 * 2     4/20/04 11:11a Rmack
 * Check in before transfer of project to John
 * 
 * 1     3/30/04 3:32p Rmack
 *
********************************************************************************
*******************************************************************************/
#ifndef _TK_PULSE_H
#define _TK_PULSE_H


// tbdrhm - Use this define to bring back the old Rev1 board resistor values 
//#define REV1_PCB
//

#define TKPULSE_INTERNAL_TRIG_RATE     10

/* Define possible drive channels */
#define TKPULSE_CHAN1            BIT0
#define TKPULSE_CHAN2            BIT1
#define TKPULSE_CHAN3            BIT2
#define TKPULSE_CHAN4            BIT3

/* Define possible trigger sources */
#define TKPULSE_TRIG1            BIT0
#define TKPULSE_TRIG2            BIT1
#define TKPULSE_TRIG3            BIT2
#define TKPULSE_TRIG4            BIT3
#define TKPULSE_SINGLE_TRIG      TKPULSE_TRIG1

#ifndef WIN32
#if (SW_BUILD_TYPE==SW_BUILD_PULSAR710)
#define TKPULSE_NUM_CHAN         4
#define TKPULSE_ALL_CHAN         (TKPULSE_CHAN1 | TKPULSE_CHAN2 | TKPULSE_CHAN3 | TKPULSE_CHAN4)
#define TKPULSE_NUM_TRIG         4
#else
#define TKPULSE_NUM_CHAN         2
#define TKPULSE_ALL_CHAN         (TKPULSE_CHAN1 | TKPULSE_CHAN2)
#define TKPULSE_NUM_TRIG         2
#endif //SW_BUILD_TYPE
#else  //WIN32
#define TKPULSE_NUM_CHAN         2
#define TKPULSE_ALL_CHAN         (TKPULSE_CHAN1 | TKPULSE_CHAN2)
#define TKPULSE_NUM_TRIG         2
#endif //WIN32



/* Define possible operating modes */
#define TKPULSE_PULSE_MODE          BIT0
#define TKPULSE_CONT_MODE           BIT1
#define TKPULSE_TIMING_BYPASS_MODE  (BIT2 | TKPULSE_PULSE_MODE)
#define TKPULSE_INTERNAL_TRIG_MODE  BIT3
#define TKPULSE_GATED_MODE          (BIT4 | TKPULSE_CONT_MODE)
#define TKPULSE_REPEAT_MODE         (BIT5 | TKPULSE_PULSE_MODE)

/* Define a flag to initiate continous mode */
#define TKPULSE_CONT_FLAG               0xCCCCCCCC

/* Define maximum pulse settings */
#define TKPULSE_MAX_WIDTH_USEC          100000
#define TKPULSE_MAX_DELAY_USEC          1000
#ifndef _WIN32
/* Define maximum current per channel */
#if(SW_BUILD_TYPE==SW_BUILD_PULSAR710)
#define TKPULSE_MAX_PULSE_CURR_AMPS     ((FLOAT32_T) 25.0)
#define TKPULSE_MAX_CONT_CURR_AMPS      ((FLOAT32_T) 2.0)
#else
#define TKPULSE_MAX_PULSE_CURR_AMPS     ((FLOAT32_T) 50.0)
#define TKPULSE_MAX_CONT_CURR_AMPS      ((FLOAT32_T) 2.0)
#endif //SW_BUILD_TYPE
#endif //_WIN32

/* Define maximum string supply voltages */
#define TKPULSE_MIN_HV_VOLTS            ((FLOAT32_T) 25.0)
#define TKPULSE_MAX_HV_VOLTS            ((FLOAT32_T) 100.0)
#define TKPULSE_MAX_LV_VOLTS            ((FLOAT32_T) 23.5)

/* Define macro to calculate DAC value for current setting */
#ifndef _WIN32
#ifdef REV1_PCB

#if (SW_BUILD_TYPE==SW_BUILD_PULSAR710)
#define TKPULSE_RSENSE_OHMS             ((FLOAT32_T) 0.5)
#define TKPULSE_SWITCHED_RSENSE_OHMS    ((FLOAT32_T) 0.1)
#define TKPULSE_CALC_PULSED_CURR_DAC_M(curr)   ((UINT16_T) ((((curr * TKPULSE_PULSE_FB_OHMS * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5) * ((100.0 + TKPULSE_DAC_ADJ_PERCENT)/100.0)))
//#define TKPULSE_CALC_PULSED_CURR_DAC_M(curr)   ((UINT16_T) ((((curr * TKPULSE_PULSE_FB_OHMS * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5)))
#define TKPULSE_CALC_CONT_CURR_DAC_M(curr)     ((UINT16_T) ((((curr * TKPULSE_CONT_FB_OHMS   * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5) * ((100.0 + TKPULSE_DAC_ADJ_PERCENT)/100.0)))
//#define TKPULSE_CALC_CONT_CURR_DAC_M(curr)     ((UINT16_T) ((((curr * TKPULSE_RSENSE_OHMS   * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5)))
#else
#define TKPULSE_RSENSE_OHMS             ((FLOAT32_T) 0.5)
#define TKPULSE_SWITCHED_RSENSE_OHMS    ((FLOAT32_T) 0.05)
#define TKPULSE_HI_RSENSE               TKPULSE_RSENSE_OHMS  
#define TKPULSE_LO_RSENSE               ((FLOAT32_T) ((TKPULSE_RSENSE_OHMS*TKPULSE_SWITCHED_RSENSE_OHMS)/(TKPULSE_RSENSE_OHMS+TKPULSE_SWITCHED_RSENSE_OHMS)))
#define TKPULSE_CALC_PULSED_CURR_DAC_M(curr)          TKPULSE_CALC_HI_RES_PULSED_CURR_DAC_M(curr)
#define TKPULSE_CALC_HI_RES_PULSED_CURR_DAC_M(curr)   ((UINT16_T) ((((curr * TKPULSE_HI_RSENSE * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5) * ((100.0 + TKPULSE_DAC_ADJ_PERCENT)/100.0)))
#define TKPULSE_CALC_LO_RES_PULSED_CURR_DAC_M(curr)   ((UINT16_T) ((((curr * TKPULSE_LO_RSENSE * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5) * ((100.0 + TKPULSE_DAC_ADJ_PERCENT)/100.0)))
//#define TKPULSE_CALC_PULSED_CURR_DAC_M(curr)   ((UINT16_T) ((((curr * TKPULSE_PULSE_FB_OHMS * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5)))
#define TKPULSE_CALC_CONT_CURR_DAC_M(curr)     TKPULSE_SUPPLY_JUMPER_ERR
//#define TKPULSE_CALC_CONT_CURR_DAC_M(curr)     ((UINT16_T) ((((curr * TKPULSE_RSENSE_OHMS   * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5)))
#define TKPULSE_HIGH_RESIST_MAX_CURR_AMPS 5.5
#endif //SW_BUILD_TYPE
#define TKPULSE_PULSE_FB_OHMS           TKPULSE_SWITCHED_RSENSE_OHMS
#define TKPULSE_CONT_FB_OHMS            TKPULSE_RSENSE_OHMS
#define TKPULSE_DAC_ADJ_PERCENT         4

#else

#if(SW_BUILD_TYPE==SW_BUILD_PULSAR710)
#define TKPULSE_RSENSE_OHMS             ((FLOAT32_T) 0.5)
#define TKPULSE_SWITCHED_RSENSE_OHMS    ((FLOAT32_T) 0.1)
#define TKPULSE_CALC_PULSED_CURR_DAC_M(curr)   ((UINT16_T) ((((curr * TKPULSE_PULSE_FB_OHMS * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5)))
#define TKPULSE_CALC_CONT_CURR_DAC_M(curr)     ((UINT16_T) ((((curr * TKPULSE_CONT_FB_OHMS  * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5)))
#else
#define TKPULSE_RSENSE_OHMS             ((FLOAT32_T) 0.5)
#define TKPULSE_SWITCHED_RSENSE_OHMS    ((FLOAT32_T) 0.05)
#define TKPULSE_HI_RSENSE               TKPULSE_RSENSE_OHMS  
#define TKPULSE_LO_RSENSE               ((FLOAT32_T) ((TKPULSE_RSENSE_OHMS*TKPULSE_SWITCHED_RSENSE_OHMS)/(TKPULSE_RSENSE_OHMS+TKPULSE_SWITCHED_RSENSE_OHMS)))
#define TKPULSE_CALC_PULSED_CURR_DAC_M(curr)          TKPULSE_CALC_HI_RES_PULSED_CURR_DAC_M(curr)
#define TKPULSE_CALC_HI_RES_PULSED_CURR_DAC_M(curr)   ((UINT16_T) ((((curr * TKPULSE_HI_RSENSE * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5)))
#define TKPULSE_CALC_LO_RES_PULSED_CURR_DAC_M(curr)   ((UINT16_T) ((((curr * TKPULSE_LO_RSENSE * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5)))
//#define TKPULSE_CALC_PULSED_CURR_DAC_M(curr)   ((UINT16_T) ((((curr * TKPULSE_PULSE_FB_OHMS * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5)))
#define TKPULSE_CALC_CONT_CURR_DAC_M(curr)     TKPULSE_CALC_PULSED_CURR_DAC_M(curr)
//#define TKPULSE_CALC_CONT_CURR_DAC_M(curr)     ((UINT16_T) ((((curr * TKPULSE_RSENSE_OHMS   * EXT_DAC_MAX_COUNTS) / EXT_DAC_VREF) + 0.5)))
#define TKPULSE_HIGH_RESIST_MAX_CURR_AMPS 5.5
#endif //SW_BUILD_TYPE
#define TKPULSE_SWITCHED_FET_OHMS       ((FLOAT32_T) 0.0124)
#define TKPULSE_PULSE_FB_OHMS           ((FLOAT32_T) (1.0/((1.0/TKPULSE_RSENSE_OHMS) + (1.0/(TKPULSE_SWITCHED_RSENSE_OHMS + TKPULSE_SWITCHED_FET_OHMS)))))
#define TKPULSE_CONT_FB_OHMS            TKPULSE_RSENSE_OHMS

#endif //REV1_PCB
#endif //_WIN32

/* Define bits for enabling drive caps */
#define TKPULSE_SMALL_CAP_EN            BIT0
#define TKPULSE_LARGE_CAP_EN            BIT1
/* Define drive-strength settings */
#define TKPULSE_SLEW_SLOW           0
#define TKPULSE_SLEW_MED_SLOW       TKPULSE_SMALL_CAP_EN
#define TKPULSE_SLEW_MED_FAST       TKPULSE_LARGE_CAP_EN
#define TKPULSE_SLEW_FAST           (TKPULSE_SMALL_CAP_EN | TKPULSE_LARGE_CAP_EN)

        

/**** Power supply feedback resistor configuration *****

                              Vsupply
                                +
                                |
                                |
                                \
                                /
                            R2  \
                                /
                                \
                                |
                         R3     |
    Vadj +-------------\/\/\/\--+------+ Vref
                                |
                                |
                                \
                                /
                            R1  \
                                /
                                \
                                |
                                |
                                +
                               GND
                                



Vadj = R3[ Vref[ 1/R1 + 1/R2 + 1/R3] - Vsupply/R2]

Vmax = R2 * Vref * [ 1/R1 + 1/R2 + 1/R3]
*********************************************************/


#define HV_VREF            ((FLOAT32_T) 1.230)
#define HV_R1              ((FLOAT32_T) 17400.0)
#define HV_R2              ((FLOAT32_T) 1000000.0)
#define HV_R3              ((FLOAT32_T) 26900.0)

#define HV_ADJ_M(vsup)     ((FLOAT32_T) HV_R3 * (HV_VREF * (1/HV_R1 + 1/HV_R2 + 1/HV_R3) - ((FLOAT32_T) vsup/HV_R2)))
#define TKPULSE_CALC_HV_DAC_M(vsup)     ((UINT8_T) ((HV_ADJ_M(vsup) * ((FLOAT32_T) DAC_MAX_COUNTS / (FLOAT32_T) DAC_VREF)) - (FLOAT32_T) 0.5))


#define LV_VREF            ((FLOAT32_T) 1.190)
#define LV_R1              ((FLOAT32_T) 88700.0)
#define LV_R2              ((FLOAT32_T) 1180000.0)
#define LV_R3              ((FLOAT32_T) 158000.0)

#define LV_ADJ_M(vsup)     ((FLOAT32_T) LV_R3 * (LV_VREF * (1/LV_R1 + 1/LV_R2 + 1/LV_R3) - ((FLOAT32_T) vsup/LV_R2)))
#define TKPULSE_CALC_LV_DAC_M(vsup)     (/* (UINT8_T) */ (UINT16_T)  ((LV_ADJ_M(vsup) * ((FLOAT32_T) DAC_MAX_COUNTS / (FLOAT32_T) DAC_VREF)) - (FLOAT32_T) 0.5))
#define TKPULSE_MAX_LV_SUPPLY    (LV_R2 * LV_VREF * (1/LV_R1 + 1/LV_R2 + 1/LV_R3))


#ifndef _WIN32

#define TKPULSE_MAX_PERIOD              IOTPU_MAX_TIME

/* Macro defines */
#define TKPULSE_INIT_M(void)              tkpulse_init(void)
#define TKPULSE_GET_ERROR_M()             tkpulse_get_error()
#define TKPULSE_SET_DATA_M(ch,w,d,c)      tkpulse_set_data(ch,w,d,c)
#define TKPULSE_GET_DATA_M(ch,wp,dp,cp)   tkpulse_get_data(ch,wp,dp,cp)
#define TKPULSE_SET_REPEAT_DATA_M(ch,t,n)      tkpulse_set_repeat_data(ch,t,n)
#define TKPULSE_GET_REPEAT_DATA_M(ch,tp,np)   tkpulse_get_repeat_data(ch,tp,np)
#define TKPULSE_SET_HV_SUPPLY_M(ch,v)     tkpulse_set_hv_supply(ch,v)
#define TKPULSE_GET_HV_SUPPLY_M(ch,vp)    tkpulse_get_hv_supply(ch,vp)
#if (SW_BUILD_TYPE==SW_BUILD_PULSAR710)
#define TKPULSE_SET_LV_SUPPLY_M(ch,v)     tkpulse_set_lv_supply(ch,v)
#define TKPULSE_GET_LV_SUPPLY_M(ch,vp)    tkpulse_get_lv_supply(ch,vp)
#else
#define TKPULSE_SET_LV_SUPPLY_M(ch,v)     NO_ERR
#define TKPULSE_GET_LV_SUPPLY_M(ch,vp)    NO_ERR
#endif
#define TKPULSE_SET_TRIGGER_MAP_M(tr,ch,r)    tkpulse_set_trigger_map(tr,ch,r,TKPULSE_PULSE_MODE)
#define TKPULSE_GET_TRIGGER_MAP_M(tr,chp,rp)  tkpulse_get_trigger_map(tr,chp,rp)
#define TKPULSE_SET_TRIGGER_DIFFERENTIAL_M(trig_state)    tkpulse_set_trigger_differential(trig_state)
#define TKPULSE_SET_GATE_MAP_M(g,ch,r)    tkpulse_set_trigger_map(g,ch,r,TKPULSE_GATED_MODE)
#define TKPULSE_TRIGGER_M(ch)             tkpulse_trigger(ch)
#define TKPULSE_SET_OUT_M(ch,s)           tkpulse_set_out(ch,s)
#define TKPULSE_GET_OUT_M(ch,sp)          tkpulse_get_out(ch,sp)
#define TKPULSE_SET_INTERNAL_TRIGGER_M(ch, r)   tkpulse_set_internal_trigger(ch, r)
#define TKPULSE_GET_INTERNAL_TRIGGER_M(chp, rp) tkpulse_get_internal_trigger(chp, rp)
#define TKPULSE_SET_MAX_FREQ_M(ch, r)     tkpulse_set_max_freq(ch, r)
#define TKPULSE_GET_MAX_FREQ_M(ch, rp)    tkpulse_get_max_freq(ch, rp)
#if (SW_BUILD_TYPE==SW_BUILD_PULSAR710)
#define TKPULSE_SET_CONTINUOUS_M(chan, curr)      tkpulse_set_continuous((chan), (curr))
#else
#define TKPULSE_SET_CONTINUOUS_M(chan, curr)      TKPULSE_BAD_TRIGGER_MODE_ERR
#endif
#define TKPULSE_SET_TIMING_BYPASS_M(ch, s) tkpulse_set_timing_bypass(ch, s)
#define TKPULSE_GET_CHAN_MODE_M(ch, mp)   tkpulse_get_chan_mode(ch, mp)
#define TKPULSE_CONTROL_LOOP_M()          tkpulse_control_loop()
#define TKPULSE_SHUTDOWN_M()              tkpulse_shutdown(TKPULSE_ALL_CHAN)
#define TKPULSE_SHUTDOWN_CHAN_M(ch)              tkpulse_shutdown(ch)
#define TKPULSE_SET_SLEW_CAPS_M(chan, value)  tkpulse_set_slew_caps(chan, value)
#define TKPULSE_GET_SLEW_CAPS_M(chan, valuep)  tkpulse_get_slew_caps(chan, valuep)
#define TKPULSE_ENABLE_TRIGGERS_M()            tkpulse_enable_triggers(TRUE)
#define TKPULSE_DISABLE_TRIGGERS_M()           tkpulse_enable_triggers(FALSE)

/* function prototypes */
ERRCODE_T tkpulse_init(void);
ERRCODE_T tkpulse_get_error(void);
ERRCODE_T tkpulse_set_data(UINT8_T chan, UINT32_T width, UINT16_T delay, FLOAT32_T current);
ERRCODE_T tkpulse_get_data(UINT8_T chan, UINT32_T *width_ptr, UINT16_T *delay_ptr, FLOAT32_T *current_ptr);
ERRCODE_T tkpulse_set_repeat_data(UINT8_T chan, UINT32_T offtime, UINT16_T num_pulses);
ERRCODE_T tkpulse_get_repeat_data(UINT8_T chan, UINT32_T* offtime_ptr, UINT16_T* num_pulses_ptr);
ERRCODE_T tkpulse_set_hv_supply(UINT8_T chan, FLOAT32_T voltage);
ERRCODE_T tkpulse_get_hv_supply(UINT8_T chan, FLOAT32_T *voltage);
ERRCODE_T tkpulse_set_lv_supply(UINT8_T chan, FLOAT32_T voltage);
ERRCODE_T tkpulse_get_lv_supply(UINT8_T chan, FLOAT32_T *voltage);
ERRCODE_T tkpulse_set_trigger_map(UINT8_T trigger, UINT8_T chan, UINT8_T rising, UINT8_T mode);
ERRCODE_T tkpulse_get_trigger_map(UINT8_T trigger, UINT8_T* chan_ptr, UINT8_T* rising_ptr);
ERRCODE_T tkpulse_set_trigger_differential(UINT8_T trigger);
ERRCODE_T tkpulse_trigger(UINT8_T chan);
ERRCODE_T tkpulse_set_out(UINT8_T chan, BOOLEAN_T state);
ERRCODE_T tkpulse_get_out(UINT8_T chan, BOOLEAN_T *state);
ERRCODE_T tkpulse_set_internal_trigger(UINT8_T chan, UINT8_T rate);
ERRCODE_T tkpulse_get_internal_trigger(UINT8_T *chan, UINT8_T *rate);
ERRCODE_T tkpulse_set_max_freq(UINT8_T chan, UINT16_T rate);
ERRCODE_T tkpulse_get_max_freq(UINT8_T chan, UINT16_T *rate);
ERRCODE_T tkpulse_set_continuous(UINT8_T chan, FLOAT32_T current);
ERRCODE_T tkpulse_set_timing_bypass(UINT8_T chan, BOOLEAN_T state);
ERRCODE_T tkpulse_get_chan_mode(UINT8_T chan, UINT8_T *mode);
ERRCODE_T tkpulse_control_loop(void);
ERRCODE_T tkpulse_shutdown(UINT8_T chan);
ERRCODE_T tkpulse_set_slew_caps(UINT8_T chan, UINT8_T value);
ERRCODE_T tkpulse_get_slew_caps(UINT8_T chan, UINT8_T* value);
ERRCODE_T tkpulse_enable_triggers(BOOLEAN_T state);

#endif





#endif /* _TK_PULSE_H */    